@@ -25,6 +25,8 @@ import gobject
 import gtk
 import Queue
 
+cups.require ("1.9.60")
+
 import authconn
 from debug import *
 import debug
@@ -77,11 +79,7 @@ class _IPPConnectionThread(threading.Thread):
         else:
             self.user = cups.getUser ()
 
-        try:
-            cups.setPasswordCB2 (self._auth)
-        except AttributeError:
-            # Requires pycups >= 1.9.47.  Fall back to rubbish API.
-            cups.setPasswordCB (self._auth)
+        cups.setPasswordCB2 (self._auth)
 
         try:
             conn = cups.Connection (host=self.host,
@@ -117,11 +115,7 @@ class _IPPConnectionThread(threading.Thread):
                 self.user = args[0]
                 cups.setUser (self.user)
                 debugprint ("Set user=%s; reconnecting..." % self.user)
-                try:
-                    cups.setPasswordCB2 (self._auth)
-                except AttributeError:
-                    # Requires pycups >= 1.9.47.  Fall back to rubbish API.
-                    cups.setPasswordCB (self._auth)
+                cups.setPasswordCB2 (self._auth)
 
                 try:
                     conn = cups.Connection (host=self.host,
@@ -166,11 +160,7 @@ class _IPPConnectionThread(threading.Thread):
         del self._auth_queue
         del conn
 
-        try:
-            cups.setPasswordCB2 (None)
-        except AttributeError:
-            # Requires pycups >= 1.9.47.  Fall back to rubbish API.
-            cups.setPasswordCB (lambda x: '')
+        cups.setPasswordCB2 (None)
 
     def _auth (self, prompt, conn=None, method=None, resource=None):
         def prompt_auth (prompt):
@@ -334,17 +324,6 @@ class _IPPAuthOperation:
         self._client_reply_handler = reply_handler
         self._client_error_handler = error_handler
 
-        if user:
-            host = conn.thread.host
-            port = conn.thread.port
-            creds = authconn.global_authinfocache.lookup_auth_info (host=host,
-                                                                    port=port)
-            if creds:
-                if creds[0] == user:
-                    self._use_password = creds[1]
-                    self._reconnected = True
-                del creds
-
         debugprint ("+%s" % self)
 
     def __del__ (self):
@@ -380,7 +359,8 @@ class _IPPAuthOperation:
         if type (exc) == cups.IPPError:
             (e, m) = exc.args
             if (e == cups.IPP_NOT_AUTHORIZED or
-                e == cups.IPP_FORBIDDEN):
+                e == cups.IPP_FORBIDDEN or
+                e == cups.IPP_AUTHENTICATION_CANCELED):
                 forbidden = (e == cups.IPP_FORBIDDEN)
             elif e == cups.IPP_SERVICE_UNAVAILABLE:
                 return self._reconnect_error (conn, exc)
@@ -411,7 +391,6 @@ class _IPPAuthOperation:
             # first.
             debugprint ("Authentication: Try as root")
             self._user = "root"
-            self._try_as_root = False
             conn.reconnect (self._user,
                             reply_handler=self._reconnect_reply,
                             error_handler=self._reconnect_error)
@@ -423,11 +402,36 @@ class _IPPAuthOperation:
             return self._error (exc)
 
         # Now reconnect and retry.
+        host = conn.thread.host
+        port = conn.thread.port
+        authconn.global_authinfocache.remove_auth_info (host=host,
+                                                        port=port)
+        self._use_password = ''
         conn.reconnect (self._user,
                         reply_handler=self._reconnect_reply,
                         error_handler=self._reconnect_error)
 
     def auth_handler (self, prompt, conn, method=None, resource=None):
+        if self._auth_called == False:
+            if self._user == None:
+                self._user = cups.getUser()
+            if self._user:
+                host = conn.thread.host
+                port = conn.thread.port
+                creds = authconn.global_authinfocache.lookup_auth_info (host=host,
+                                                                        port=port)
+                if creds:
+                    if creds[0] == self._user:
+                        self._use_password = creds[1]
+                        self._reconnected = True
+                    del creds
+        else:
+            host = conn.thread.host
+            port = conn.thread.port
+            authconn.global_authinfocache.remove_auth_info (host=host,
+                                                            port=port)
+            self._use_password = ''
+
         self._auth_called = True
         if self._reconnected:
             debugprint ("Supplying password after reconnection")
@@ -485,6 +489,12 @@ class _IPPAuthOperation:
 
     def _on_auth_dialog_response (self, dialog, response):
         (user, password) = dialog.get_auth_info ()
+        if user == '':
+            user = self._user;
+        authconn.global_authinfocache.cache_auth_info ((user,
+                                                        password),
+                                                       host=self._conn.thread.host,
+                                                       port=self._conn.thread.port)
         self._dialog = dialog
         dialog.hide ()
 
@@ -492,6 +502,8 @@ class _IPPAuthOperation:
             response == gtk.RESPONSE_DELETE_EVENT):
             self._cancel = True
             self._conn.set_auth_info ('')
+            authconn.global_authinfocache.remove_auth_info (host=self._conn.thread.host,
+                                                            port=self._conn.thread.port)
             debugprint ("Auth canceled")
             return
 
